import { inLine, inArc, inBox, inRect } from './utils/math';
export function isPointInPath(displayObject, position, isPointInPath) {
  var _a = displayObject.parsedStyle,
      radius = _a.radius,
      fill = _a.fill,
      stroke = _a.stroke,
      _b = _a.lineWidth,
      lineWidth = _b === void 0 ? 0 : _b,
      width = _a.width,
      height = _a.height,
      clipPathTargets = _a.clipPathTargets;
  var isClipPath = !!(clipPathTargets === null || clipPathTargets === void 0 ? void 0 : clipPathTargets.length); // 无圆角时的策略

  if (!radius) {
    var halfWidth = lineWidth / 2; // 同时填充和带有边框

    if (fill && stroke || isClipPath) {
      return inBox(0 - halfWidth, 0 - halfWidth, width + halfWidth, height + halfWidth, position.x, position.y);
    } // 仅填充


    if (fill) {
      return inBox(0, 0, width, height, position.x, position.y);
    }

    if (stroke) {
      return inRect(0, 0, width, height, lineWidth, position.x, position.y);
    }
  } else {
    var isHit = false;

    if (stroke || isClipPath) {
      isHit = inRectWithRadius(0, 0, width, height, radius, lineWidth, position.x, position.y);
    } // 仅填充时带有圆角的矩形直接通过图形拾取
    // 以后可以改成纯数学的近似拾取，将圆弧切割成多边形


    if (!isHit && (fill || isClipPath)) {
      isHit = isPointInPath(displayObject, position);
    }

    return isHit;
  }

  return false;
}

function inRectWithRadius(minX, minY, width, height, radius, lineWidth, x, y) {
  return inLine(minX + radius, minY, minX + width - radius, minY, lineWidth, x, y) || inLine(minX + width, minY + radius, minX + width, minY + height - radius, lineWidth, x, y) || inLine(minX + width - radius, minY + height, minX + radius, minY + height, lineWidth, x, y) || inLine(minX, minY + height - radius, minX, minY + radius, lineWidth, x, y) || inArc(minX + width - radius, minY + radius, radius, 1.5 * Math.PI, 2 * Math.PI, lineWidth, x, y) || inArc(minX + width - radius, minY + height - radius, radius, 0, 0.5 * Math.PI, lineWidth, x, y) || inArc(minX + radius, minY + height - radius, radius, 0.5 * Math.PI, Math.PI, lineWidth, x, y) || inArc(minX + radius, minY + radius, radius, Math.PI, 1.5 * Math.PI, lineWidth, x, y);
}