import { SHAPE, RenderingPluginContribution } from '@antv/g';
import { ContainerModule } from 'inversify';
import { CanvasPickerPlugin, PointInPathPickerFactory } from './CanvasPickerPlugin';
import { isPointInPath as CirclePicker } from './Circle';
import { isPointInPath as EllipsePicker } from './Ellipse';
import { isPointInPath as LinePicker } from './Line';
import { isPointInPath as PolylinePicker } from './Polyline';
import { isPointInPath as PathPicker } from './Path';
import { isPointInPath as RectPicker } from './Rect';
import { isPointInPath as PolygonPicker } from './Polygon';
var containerModule = new ContainerModule(function (bind, unbind, isBound, rebind) {
  bind(PointInPathPickerFactory).toFactory(function (ctx) {
    return function (tagName) {
      if (tagName === SHAPE.Circle) {
        return CirclePicker;
      } else if (tagName === SHAPE.Ellipse) {
        return EllipsePicker;
      } else if (tagName === SHAPE.Rect) {
        return RectPicker;
      } else if (tagName === SHAPE.Line) {
        return LinePicker;
      } else if (tagName === SHAPE.Polyline) {
        return PolylinePicker;
      } else if (tagName === SHAPE.Polygon) {
        return PolygonPicker;
      } else if (tagName === SHAPE.Path) {
        return PathPicker;
      } else if (tagName === SHAPE.Image || tagName === SHAPE.Text // tagName === SHAPE.Group
      ) {
        return function () {
          return true;
        };
      }

      return null;
    };
  });
  bind(CanvasPickerPlugin).toSelf().inSingletonScope();
  bind(RenderingPluginContribution).toService(CanvasPickerPlugin);
});

var Plugin = function () {
  function Plugin() {}

  Plugin.prototype.init = function (container) {
    container.load(containerModule);
  };

  Plugin.prototype.destroy = function (container) {
    container.unload(containerModule);
  };

  return Plugin;
}();

export { Plugin };